==> gmssl-sdf-crypto-1.1.3-20210703.101241-1-sources.jar: com/xdja/pki/gmssl/crypto/sdf/SdfContentOutputStream.java <==
package com.xdja.pki.gmssl.crypto.sdf;

import com.xdja.pki.gmssl.core.utils.GMSSLBCSignUtils;
import com.xdja.pki.gmssl.core.utils.GMSSLByteArrayUtils;
import com.xdja.pki.gmssl.core.utils.GMSSLECUtils;
import com.xdja.pki.gmssl.sdf.SdfSDKException;
import org.bouncycastle.asn1.gm.GMObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
import org.bouncycastle.operator.RuntimeOperatorException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.SignatureException;

public class SdfContentOutputStream extends OutputStream {

    private SdfSigner signer;
    private Exception exception;
    private String ecdasSignName = "SHA256WITHECDSA";
    private Logger logger = LoggerFactory.getLogger(this.getClass());
    private byte[] data = new byte[0];

    SdfContentOutputStream(SdfCryptoType sdfCryptoType, AlgorithmIdentifier algorithmIdentifier, boolean forSigning, AsymmetricKeyParameter asymmetricKeyParameter) {
        this(sdfCryptoType, algorithmIdentifier, forSigning, asymmetricKeyParameter, Constants.SDF_SM2_USED_BC_SM3);
    }

    SdfContentOutputStream(SdfCryptoType sdfCryptoType, AlgorithmIdentifier algorithmIdentifier, boolean forSigning, AsymmetricKeyParameter asymmetricKeyParameter, boolean useBC) {
        try {
            if (SdfSHAType.checkSupportType(algorithmIdentifier.getAlgorithm())) {
                this.signer = new SdfRSASigner(sdfCryptoType, algorithmIdentifier);
            } else if (GMObjectIdentifiers.sm2sign_with_sm3.getId().equals(algorithmIdentifier.getAlgorithm().getId())) {
                if (useBC) {
                    this.signer = new SdfSM2SignerSM3(sdfCryptoType);
                } else {
                    this.signer = new SdfECSigner(sdfCryptoType);
                }
            } else if (X9ObjectIdentifiers.ecdsa_with_SHA256.getId().equals(algorithmIdentifier.getAlgorithm().getId())) {
                this.signer = new SdfECSigner(sdfCryptoType, ecdasSignName);
            } else {
                throw new SdfSDKException("UnSupport algorithmIdentifier: " + algorithmIdentifier.getAlgorithm().getId());
            }
            this.signer.init(forSigning, asymmetricKeyParameter);
        } catch (SdfSDKException e) {
            throw new RuntimeOperatorException("exception obtaining signature init : " + e.getMessage(), e);
        }
    }

    @Override
    public void write(byte[] bytes, int off, int len) {
        byte[] tmp = new byte[len];
        System.arraycopy(bytes, off, tmp, 0, len);
        data = GMSSLByteArrayUtils.mergeByteArray(data, tmp);
    }

    @Override
    public void write(byte[] bytes) {
        data = GMSSLByteArrayUtils.mergeByteArray(data, bytes);
    }

    @Override
    public void write(int var1) {
        byte[] tmp = new byte[1];
        tmp[0] = (byte) var1;
        data = GMSSLByteArrayUtils.mergeByteArray(data, tmp);
    }

    byte[] getSignature() throws SignatureException {
        try {
            this.signer.update(data, 0, data.length);
            this.data = new byte[0];
            byte[] signature = signer.generateSignature();
            signer.release();
            return signature;
        } catch (CryptoException | SdfSDKException e) {
            throw new SignatureException("exception obtaining signature: " + e.getMessage(), e);
        }
    }

    boolean verify(byte[] expected) throws SignatureException {
        try {
            this.signer.update(data, 0, data.length);
            this.data = new byte[0];
            boolean verify = signer.verifySignature(expected);
            signer.release();
            return verify;
        } catch (SdfSDKException e) {
            throw new SignatureException("exception obtaining signature: " + e.getMessage(), e);
        }
    }

}

==> gmssl-sdf-crypto-1.3.2-20220127.034310-1-sources.jar: com/xdja/pki/gmssl/crypto/sdf/SdfContentOutputStream.java <==
package com.xdja.pki.gmssl.crypto.sdf;

import com.xdja.pki.gmssl.core.utils.GMSSLByteArrayUtils;
import com.xdja.pki.gmssl.sdf.SdfSDKException;
import org.bouncycastle.asn1.gm.GMObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
import org.bouncycastle.operator.DefaultAlgorithmNameFinder;
import org.bouncycastle.operator.RuntimeOperatorException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.OutputStream;
import java.security.SignatureException;

public class SdfContentOutputStream extends OutputStream {

    private SdfSigner signer;
    private Exception exception;
    private Logger logger = LoggerFactory.getLogger(this.getClass());
    private byte[] data = new byte[0];

    SdfContentOutputStream(SdfCryptoType sdfCryptoType, AlgorithmIdentifier algorithmIdentifier, boolean forSigning, AsymmetricKeyParameter asymmetricKeyParameter) {
        this(sdfCryptoType, algorithmIdentifier, forSigning, asymmetricKeyParameter, Constants.SDF_SM2_USED_BC_SM3);
    }

    SdfContentOutputStream(SdfCryptoType sdfCryptoType, AlgorithmIdentifier algorithmIdentifier, boolean forSigning, AsymmetricKeyParameter asymmetricKeyParameter, boolean useBC) {
        this(sdfCryptoType, algorithmIdentifier, forSigning, asymmetricKeyParameter, useBC, 256);

    }

    SdfContentOutputStream(SdfCryptoType sdfCryptoType, AlgorithmIdentifier algorithmIdentifier, boolean forSigning, AsymmetricKeyParameter asymmetricKeyParameter, boolean useBC, int bits) {
        try {
            if (SdfSHAType.checkSupportType(algorithmIdentifier.getAlgorithm())) {
                this.signer = new SdfRSASigner(sdfCryptoType, algorithmIdentifier);
            } else if (GMObjectIdentifiers.sm2sign_with_sm3.getId().equals(algorithmIdentifier.getAlgorithm().getId())) {
                if (useBC) {
                    this.signer = new SdfEcSignerWithSoftDigest(sdfCryptoType);
                } else {
                    this.signer = new SdfECSigner(sdfCryptoType);
                }
            } else {
                String algorithmName = new DefaultAlgorithmNameFinder().getAlgorithmName(algorithmIdentifier);
                if (useBC) {
                    this.signer = new SdfEcSignerWithSoftDigest(sdfCryptoType, algorithmName, bits);
                } else {
                    this.signer = new SdfECSigner(sdfCryptoType, algorithmName, bits);
                }
            }
            this.signer.init(forSigning, asymmetricKeyParameter);
        } catch (SdfSDKException e) {
            throw new RuntimeOperatorException("exception obtaining signature init : " + e.getMessage(), e);
        }
    }

    @Override
    public void write(byte[] bytes, int off, int len) {
        byte[] tmp = new byte[len];
        System.arraycopy(bytes, off, tmp, 0, len);
        data = GMSSLByteArrayUtils.mergeByteArray(data, tmp);
    }

    @Override
    public void write(byte[] bytes) {
        data = GMSSLByteArrayUtils.mergeByteArray(data, bytes);
    }

    @Override
    public void write(int var1) {
        byte[] tmp = new byte[1];
        tmp[0] = (byte) var1;
        data = GMSSLByteArrayUtils.mergeByteArray(data, tmp);
    }

    byte[] getSignature() throws SignatureException {
        try {
            this.signer.update(data, 0, data.length);
            this.data = new byte[0];
            byte[] signature = signer.generateSignature();
            signer.release();
            return signature;
        } catch (CryptoException | SdfSDKException e) {
            throw new SignatureException("exception obtaining signature: " + e.getMessage(), e);
        }
    }

    boolean verify(byte[] expected) throws SignatureException {
        try {
            this.signer.update(data, 0, data.length);
            this.data = new byte[0];
            boolean verify = signer.verifySignature(expected);
            signer.release();
            return verify;
        } catch (SdfSDKException e) {
            throw new SignatureException("exception obtaining signature: " + e.getMessage(), e);
        }
    }

}
